<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Automatically assess the speech from a recorded video by extracting the
 * audio, sending it to the Google speech-to-text API, and assessing the
 * results against the criteria set for the assignment.
 *
 * @package   assignfeedback_vwttext
 * @copyright 2023 Bokomaru Publications
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace assignfeedback_vwttext;

use context_module;
use assign;

class assessor {
    /**
     * Grade submission and provide feedback by:
     *   1. getting the contents of the assignsubmission_onlinetext
     *      submission;
     *   2. Sending the submission to the VWT API;
     *   3. Populating an assignfeedback_comments instance with the API's
     *      feedback.
     *   4. Setting the grade to the score provided by the API.
     * @param int $submissionid ID of the submission.
     * @param int $contextid Context ID of the mod_assign instance.
     * @param int $userid ID of user who's submission this is.
     */
    public function assess($submissionid, $contextid, $userid) {
        global $DB;

        $context = \context::instance_by_id($contextid);
        $assign = new assign($context, null, null);
        $submissionvwttext = new \assign_submission_vwttext($assign, 'vwttext');
        $testlink = $submissionvwttext->get_config('testlink');
        $submissiononlinetext = new \assign_submission_onlinetext($assign, 'onlinetext');
        $text = $submissiononlinetext->get_editor_text('onlinetext', $submissionid);
        $text = html_to_text($text, 0, false);

        $vwt = new \assignsubmission_vwttext\vwt_api();
        $feedback = $vwt->feedback($testlink, $text);
        $result = $feedback['result'];
        $score = self::to_grade($feedback);

        $assignmentid = $assign->get_instance()->id;
        $feedbackcomments = null;
        $grade = $assign->get_user_grade($userid, true);

        if ($grade) {
            $feedbackcomments = $DB->get_record('assignfeedback_comments', array('grade' => $grade->id), '*', IGNORE_MISSING);
        }

        if ($feedbackcomments) {
            $feedbackcomments->commenttext = $result;
            $feedbackcomments->commentformat = FORMAT_HTML;
            $DB->update_record('assignfeedback_comments', $feedbackcomments);
        } else {
            $feedbackcomments = new \stdClass();
            $feedbackcomments->assignment = $assignmentid;
            $feedbackcomments->grade = $grade->id;
            $feedbackcomments->commenttext = $result;
            $feedbackcomments->commentformat = FORMAT_HTML;
            $DB->insert_record('assignfeedback_comments', $feedbackcomments);
        }

        // The "Needs grading" count on the "Grading summary" page relies on
        // mdl_assign_grades.timemodified being greater than
        // mdl_assign_submission.timemodified (see MDL-48861).  Since this
        // plugin grades the submission upon save these timestamps could be
        // equal for a submission < 1 second.  So we sleep for 1 second to
        // ensure the former timestamp is greater than the latter.
        sleep(1);

        $grade->grade = $score;
        $assign->update_grade($grade);
    }

    /**
     * Given the feedback from the VWT API for an essay return a grade value.
     * The VWT API returns a score which can be below 0, and a max_score.
     * If score is below 0 then we return 0.  If score is greater than 0 we
     * return this value as a percentage of max_score (for example if score
     * is 10 and max_score is 20 we return 50).
     * @param array $feedback Array with elements 'score' and 'max_score'.
     * @return float Value between 0 and 100.
     */
    public static function to_grade(array $feedback): float {
        $score = (max(0, $feedback['score']) / $feedback['max_score']) * 100;

        return round($score, 5);
    }
}
