<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The assignfeedback_vwttext failure of automatic grading event.
 *
 * @package    assignfeedback_vwttext
 * @copyright  2023 Bokomaru Publications
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace assignfeedback_vwttext\event;

use context_module;
use assign;

/**
 * The assignfeedback_vwttext automatic grading failure event class.
 * @property-read array $other {
 *      Extra information about event.
 *
 *      - string message: Any error message that caused the failure to grade.
 * }
 * @package    assignfeedback_vwttext
 * @copyright  2023 Bokomaru Publications
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class grading_failed extends \mod_assign\event\base {

    /**
     * Create instance of event.  This is intended to be called from an
     * assessable_submitted observer when the grading has failed for some
     * reason (e.g. failure during API call).
     * @param object assessable_submitted A mod_assign assessable_submitted
     * event.
     * @param string $error Some error details (e.g. any error returned by API
     * during grading).
     * @return object grading_failed instance.
     */
    public static function create_from_assessable_submitted(\mod_assign\event\assessable_submitted $assessablesubmitted, string $message) {
        global $DB;

        $submission = $DB->get_record('assign_submission', array('id' => $assessablesubmitted->objectid), '*', MUST_EXIST);
        $context = \context::instance_by_id($assessablesubmitted->contextid);

        $data = array(
            'context' => $context,
            'objectid' => $assessablesubmitted->objectid,
            'other' => array(
                'message' => $message,
            ),
        );

        $data['relateduserid'] = $assessablesubmitted->relateduserid;

        $event = self::create($data);
        $event->set_assign($assessablesubmitted->get_assign());
        $event->add_record_snapshot('assign_submission', $submission);

        return $event;
    }

    /**
     * Returns description of what happened.
     *
     * @return string
     */
    public function get_description() {
        return "Failed to grade submission '$this->objectid' " .
            "for the assignment with course module id '$this->contextinstanceid'." .
            " Details: {$this->other['message']}";
    }

    /**
     * Return localised event name.
     *
     * @return string
     */
    public static function get_name() {
        return get_string('eventgradingfailed', 'assignfeedback_vwttext');
    }

    /**
     * Init method.
     *
     * @return void
     */
    protected function init() {
        $this->data['objecttable'] = 'assign_submission';
        $this->data['crud'] = 'u';
        $this->data['edulevel'] = self::LEVEL_PARTICIPATING;
    }

    public static function get_objectid_mapping() {
        return array('db' => 'assign_submission', 'restore' => 'submission');
    }

    public static function get_other_mapping() {
        // Nothing to map.
        return false;
    }
}
