<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @package assignfeedback_vwttext
 * @copyright 2023 Bokomaru Publications
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

class assign_feedback_vwttext extends assign_feedback_plugin {
    const COMPONENT = 'assignfeedback_vwttext';

    /**
     * Get the name of the this submission plugin
     * @return string
     */
    public function get_name() {
        return get_string('vwttext', self::COMPONENT);
    }

    /**
     * Get form elements for grading form.
     * @param stdClass $grade
     * @param MoodleQuickForm $mform
     * @param stdClass $data
     * @param int $userid Userid we are currently grading
     * @return bool True if elements were added to the form
     */
    public function get_form_elements_for_user($grade, MoodleQuickForm $mform, stdClass $data, $userid) {
        if (!\assignsubmission_vwttext\vwt_api::available()) {
            $name = self::COMPONENT . "_warning";
            $settingsurl = new moodle_url('/admin/settings.php',
                        array('section' => 'assignsubmission_vwttext'));
            $mform->addElement('static', $name, '',
                        get_string('missingvwtkey', 'assignsubmission_vwttext',
                                    (string) $settingsurl));
            return true;
        }

        global $PAGE, $DB;

        $cm = get_coursemodule_from_instance('assign', $grade->assignment, 0, false, MUST_EXIST);
        $context = context_module::instance($cm->id);
        $assign = new assign($context, null, null);
        $submissionvwttext = new assign_submission_vwttext($assign, 'vwttext');
        $submissiononlinetext = new assign_submission_onlinetext($assign, 'onlinetext');
        $submission = $this->assignment->get_user_submission($grade->userid, false, -1);
        $testlink = $submissionvwttext->get_config('testlink');
        $attributes = array();

        if ($submissiononlinetext->is_empty($submission)) {
            $attributes['disabled'] = '';
        }

        global $CFG;
        $texteditors = explode(',', $CFG->texteditors);
        $htmleditor = get_user_preferences('htmleditor', $texteditors[0]);
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/editor', 'init', [$htmleditor]);
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/evaluate');

        $attributes['onclick'] = "javascript:require('assignsubmission_vwttext/evaluate').grade('$testlink');";
        $name = self::COMPONENT . '_evaluate';
        $mform->addElement('button', $name, get_string('automatedevaluation', 'assignfeedback_vwttext'), $attributes);

        if (\assignsubmission_vwttext\gptzero_api::available()) {
            $attributes['onclick'] = "javascript:require('assignsubmission_vwttext/evaluate').gptzero();";
            $name = self::COMPONENT . '_gptzero';
            $mform->addElement('button', $name, get_string('checkforgpt', 'assignfeedback_vwttext'), $attributes);
        }

        $name = self::COMPONENT . '_loading';
        $mform->addElement('html', '<div name="' . $name . '"></div>');

        return true;
    }

    /**
     * Has the feedback been modified?  Returns false as no feedback is set by
     * this plugin (instead it populates assignfeedback_comments' feedback).
     * @param stdClass $grade Grade object.
     * @param stdClass $data Form data.
     * @return boolean False.
     */
    public function is_feedback_modified(stdClass $grade, stdClass $data) {
        return false;
    }
}
